# Floppy boot segment

.org 0# actually 7c00
start:	jmp start0
	nop 

	.ascii "cmcf 1.0"
	.word 512# bytes/sector
	.byte 1# sector/cluster
	.word 1# sector reserved
	.byte 2# FATs
	.word 16 * 14# root directory entries
	.word 80 * 2 * 18# sectors
	.byte 0x0F0# media
	.word 9# sectors/FAT
	.word 18# sectors/track
	.word 2# heads
	.int 0# hidden sectors
	.int 80 * 2 * 18# sectors again
	.byte 0# drive
#    db 0
#    db 29h     ; signature
#    dd 44444444h     ; serial
#    db 'COLOR FORTH' ; label
#    db '        '

command: 	.byte 0
	.byte 0# head, drive
cylinder: 	.byte 0
	.byte 0# head
	.byte 1# sector
	.byte 2# 512 bytes/sector
	.byte 18# sectors/track
	.byte 0x1b# gap
	.byte 0x0ff
.balign 4
nc: 	.int 9# Forth+Icons+blocks 24-161
gdt: 	.word 0x17
	.int gdt0
.balign 8
gdt0: 	.word 0, 0, 0, 0
	.word 0x0FFFF, 0, 0x9A00, 0x0CF# code
	.word 0x0FFFF, 0, 0x9200, 0x0CF# data

# Code is compiled in Protected 32-bit mode.
# Hence  org $-2  to fix 16-bit words
# and 4 hand-assembled instructions.
# and EAX and AX exchanged
# This code is in Real 16-bit mode
.code16
start0:	mov $0x4F02, %ax# Video mode
	mov $vesa, %bx# hp*vp rgb: 565
	int $0x10
	cli 
	xor %eax, %eax# Move code to 0
	mov %eax, %ebx
	mov %cs, %bx
	mov %bx, %ds
	mov %ax, %es
	movl %eax, %edi
	movl %eax, %esi
	call loc# Where are we? IP+4*CS
loc:	pop %si
	sub $loc - start, %si
	mov $512 / 4, %cx
	rep
	movsl 
	jmp  $0, $relocate
#	.byte 0x0ea
#	.word relocate - start, 0

relocate:# This code is executed from 0
	mov %ax, %ds
	lgdt gdt
#	.byte 0x0f, 1, 0x16
#	.word gdt - start
	movb $1, %al
	movl %eax, %cr0
	jmp  $8, $protected
#	.byte 0x0ea
#	.word protected - start, 8
.code32
protected:# Now in Protected 32-bit mode
	movb $0x10, %al
	mov %eax, %ds
	mov %eax, %es
	mov %eax, %ss
	movl $gods, %esp
	xorl %ecx, %ecx

a20:	movb $0x0d1, %al
	outb %al, $0x64
0:	inb $0x64, %al
	andb $2, %al
	jnz 0b
	movb $0x4b, %al
	outb %al, $0x60

	call dma
	shll $4, %ebx
	addl %ebx, %esi
	cmpl $0x44444444, (%esi)# Boot?
	jnz cold
	movw $63 * 0x100 - 0x80, %cx# Nope
	rep
	movsl 
	movl $godd, %esi
	jmp start2

cold:	call sense_
	jns cold
	movl $godd, %esi
	xorl %edi, %edi# Cylinder 0 on top of Address 0
	movb nc, %cl
0:	pushl %ecx
	call read
	incb cylinder
	popl %ecx
	loop 0b
start2:	call stop
	jmp start1

.equ us, 1000 / 6 
.equ ms, 1000 * us 
spin:	movb $0x1c, %cl
	call onoff
#    mov  DX, 3f2h
#    out  DX, AL
0:	call sense_
	jns 0b
	movb $0, cylinder# calibrate
	movb $7, %al
	movb $2, %cl
	call cmd
	movl $500 * ms, %ecx
0:	loop 0b
cmdi:	call sense_
	js cmdi
	ret 

ready:#call delay
	movw $0x3f4, %dx
0:	in %dx, %al
	outb %al, $0x0e1
	shlb $1, %al
	jnc 0b
	leal 1(%edx), %edx
	ret 

transfer:	movb $9, %cl
cmd:	lea command, %edx
	movb %al, (%edx)
cmd0:	pushl %esi
	movl %edx, %esi
cmd1:	call ready
	jns 0f
	in %dx, %al
	jmp cmd1
0:	lodsb 
	out %al, %dx
	outb %al, $0x0e1
	loop cmd1
	popl %esi
#delay: mov  EAX, us
#@@:     dec  EAX
#        jnz  @b
	ret 

sense_:	movb $8, %al
	movl $1, %ecx
	call cmd
0:	call ready
	jns 0b
	in %dx, %al
	outb %al, $0x0e1
	andb %al, %al
#    cmp  AL, 80h
	ret 

seek:	call sense_
	jns seek
	ret 

stop:	movb $0x0c, %cl# Motor off
onoff:	DUP_ 
	movb %cl, %al
	movw $0x3f2, %dx
	out %al, %dx
	outb %al, $0x0e1
	DROP 
	ret 

dma:	movw $0x3a2, command + 1# l2 s6 u32 ms (e 2)
	movb $3, %al# timing
	movb $3, %cl
	call cmd
	movw $0x7000, command + 1# +seek -fifo -poll
	movb $0x13, %al# configure
	movb $4, %cl
	call cmd
	movl %ecx, command# 0
	ret 

read:	call seek
	movb $0x0e6, %al# Read normal data
	call transfer
	movw $18 * 2 * 512, %cx
0:	call ready
	in %dx, %al
	outb %al, $0x0e1
	stosb 
	next 0b
	ret 

write:	call seek
	movb $0x0c5, %al# Write data
	call transfer
	movw $18 * 2 * 512, %cx
0:	call ready
	lodsb 
	out %al, %dx
	outb %al, $0x0e1
	next 0b
	ret 

.org 0x1fe# Mark boot sector
	.word 0x0aa55
	.int 0x44444444# Mark COLOR.COM

flop:	movb %al, cylinder# c-cx
	DUP_ 
	movw $0x3f2, %dx
	in %dx, %al
	outb %al, $0x0e1
	testb $0x10, %al
	jnz 0f
	jmp spin
0:	ret 

readf:	call flop# ac-ac
	pushl %edi
	movl 4(%esi), %edi
	shll $2, %edi
	call read
	popl %edi
readf1:	DROP 
	incl %eax
	addl $0x1200, (%esi)
	ret 

writef:	call flop# ac-ac
	pushl %esi
	movl 4(%esi), %esi
	shll $2, %esi
	call write
	popl %esi
	jmp readf1

seekf:	call flop# c-c
#    call delay
	call seek
	movb $0x0f, %al
	movb $3, %cl
	call cmd
	call cmdi
	DROP 
	ret 

cmdf:	movl %eax, %ecx# an
	DROP 
	leal (, %eax, 4), %edx
	call cmd0
	DROP 
	ret 

readyf:	DUP_ 
	call ready
	DROP 
	ret 
